package de.uni_marburg.powersort;

import de.uni_marburg.powersort.benchmark.DummyComparable1;
import de.uni_marburg.powersort.benchmark.NaturalOrder;

import java.util.Arrays;

/**
 * Quick intro on how sorting arrays works in JDK 23.
 */
public class Educational {
    public static void main(String[] args) {
        sortObjectsAvoidingComparableTimSort();
        sortObjects();
        sortPrimitives();
    }

    /**
     * TODO: If this is no slower than sortObjects(), then we could avoid ComparableTimSort :)
     * <p>
     * Quote from JDK23 ComparableTimSort.java
     * <p>
     * If you are using an optimizing VM, you may find that ComparableTimSort
     * offers no performance benefit over TimSort in conjunction with a
     * comparator that simply returns {@code ((Comparable)first).compareTo(Second)}.
     * If this is the case, you are better off deleting ComparableTimSort to
     * eliminate the code duplication.
     */
    private static void sortObjectsAvoidingComparableTimSort() {
        DummyComparable1[] a = {
                new DummyComparable1(1),
                new DummyComparable1(0)
        };

        NaturalOrder c = NaturalOrder.INSTANCE;
        // -> TimSort.sort(a, 0, a.length, c, null, 0, 0);
        // -> All elements in the array must implement the {@link Comparable} interface.
        Arrays.sort(a, c);

        System.out.println(Arrays.toString(a));
    }

    public static void sortObjects() {
        DummyComparable1[] a = {
                new DummyComparable1(1),
                new DummyComparable1(0)
        };


        // -> ComparableTimSort.sort(a, 0, a.length, null, 0, 0);
        // -> All elements in the array must implement the {@link Comparable} interface.
        Arrays.sort(a);

        System.out.println(Arrays.toString(a));
    }

    public static void sortPrimitives() {
        int[] a = {1, 0};

        // -> DualPivotQuicksort.sort(a, 0, 0, a.length);
        Arrays.sort(a);

        System.out.println(Arrays.toString(a));
    }
}
