package de.uni_marburg.powersort.benchmark;

import de.uni_marburg.powersort.data.DataEnum;
import de.uni_marburg.powersort.sort.SortEnum;
import de.uni_marburg.powersort.data.ObjectSupplier;

import java.util.EnumSet;
import java.util.concurrent.TimeUnit;

/**
 * Custom benchmark.
 */
public class Main {
    public static void main(final String[] args) {
        final EnumSet<SortEnum> sortImplementations = getSortImplementations();
        final EnumSet<DataEnum> dataEnums = getSortInputSuppliers();

        for (DataEnum dataEnum : dataEnums) {
            ObjectSupplier objectSupplier = dataEnum.get();
            System.out.println("\n" + objectSupplier.title());

            for (SortEnum sortImplementation : sortImplementations) {
                Object[] sortInput = objectSupplier.getCopy();

                // TODO: JVM warmup!
                final long startNanos = System.nanoTime();
                sortImplementation.get().sort(sortInput);
                final long stopNanos = System.nanoTime();

                final long durNanos = stopNanos - startNanos;
                final long durMillis = TimeUnit.NANOSECONDS.toMillis(durNanos);
                final String durFormatted = LongFormatter.formatUnderscore(durMillis);
                System.out.println(durFormatted + "," + sortImplementation);
            }
        }
    }

    static EnumSet<SortEnum> getSortImplementations() {
        return EnumSet.allOf(SortEnum.class);
    }

    /**
     * The returned ObjectSupplier objects are wrapped by DataEnum objects.
     * This way they are lazily created on their first access with DataEnum.get().
     * This saves memory if we work with large lists.
     */
    static EnumSet<DataEnum> getSortInputSuppliers() {
        return EnumSet.allOf(DataEnum.class);
    }
}
