package de.uni_marburg.powersort.benchmark;

import java.util.Random;

/**
 * Provides utility methods related to random integers.
 */
public final class RandomInt {
  private RandomInt() {
  }

  /**
   * @return A random integer.
   */
  public static int integer() {
    return integer(Integer.MIN_VALUE, Integer.MAX_VALUE);
  }
  public static int integer(final int minInt, final int maxInt) {
    final double rand01 = Math.random();
    return helper(minInt, maxInt, rand01);
  }

  /**
   * The returned random integer to return is determined by the given seed.
   *
   * @return A random integer.
   */
  public static int integer(final long seed) {
    Random random = new Random(seed);
    final double rand01 = random.nextDouble();
    return helper(Integer.MIN_VALUE, Integer.MAX_VALUE, rand01);

  }

  /**
   * @param rand01 Random value in range [0,1)
   */
  private static int helper(final int minInt, final int maxInt, final double rand01) {
    final double randMinMax = rand01 * (maxInt - minInt) - minInt;
    return (int) Math.round(randMinMax);
  }
}
