package de.uni_marburg.powersort.sort;

import de.uni_marburg.powersort.JUnitUtil;
import de.uni_marburg.powersort.data.AscendingRuns;
import de.uni_marburg.powersort.data.DescendingIntegers;
import org.junit.jupiter.params.ParameterizedTest;
import org.junit.jupiter.params.converter.ConvertWith;
import org.junit.jupiter.params.provider.CsvSource;

import java.util.Arrays;

import static org.junit.jupiter.api.Assertions.assertArrayEquals;

public abstract class AbstractSortTest {
    SortEnum sortAlg;

    @ParameterizedTest
    @CsvSource({
            "''",
            "1337",
            "|2|3|1",
            "1|1",
            "2|1|2",
    })
    void test1(@ConvertWith(JUnitUtil.IntegerArrayConverter.class) Integer[] array) {
        sortAndCheckResult(array);
    }

    @ParameterizedTest
    @CsvSource({
            "3,7,-13",
            "3,7,-3",
    })
    void test2(int numOfRuns, int runLength, int decreaseBetweenRuns) {
        Integer[] array = AscendingRuns.newAscendingRuns(numOfRuns, runLength, decreaseBetweenRuns).getCopy();
        sortAndCheckResult(array);
    }
    @ParameterizedTest
    @CsvSource({
            "2",
            "3",
            "13",
            "1337",
    })
    void test2(int size) {
        Integer[] array = new DescendingIntegers(size).getCopy();
        sortAndCheckResult(array);
    }

    void sortAndCheckResult(Integer[] array){
        Integer[] expected = Arrays.copyOf(array, array.length);
        Arrays.sort(expected);

        sortAlg.get().sort(array);
        assertArrayEquals(expected, array);
    }
}
